#***************************************************************************************************
#  read initial values from CarbonTracker file and attach them to the "result" matrix
#***************************************************************************************************

getCarbonTracker <- function(yr4=NULL, mon=NULL, day=NULL, hr=NULL, co2inifile=NULL,
                             result=NULL, result.sel=NULL) {

# --------------------------------------------------------------------------------------------------
#  Interface
#  =========
#
#  yr4          YYYY of the receptor time
#  mon          month "
#  day          day   "
#  hr           hour  "
#  co2inifile   netCDF file name (absolute path) of the CarbonTracker data
#  result       matrix with columns "btime", "lat", "lon" and "pres"
#               gets the column "co2ini" attached/overwritten on output
#  result.sel   selector of same length as the columns of "result": which rows should be considered
#
# --------------------------------------------------------------------------------------------------
#  $Id: getCarbonTracker.r,v 1.4 2007/09/20 15:55:53 trn Exp $
# --------------------------------------------------------------------------------------------------

   # CarbonTracker notes
   if (!is.element("package:ncdf", search())) library("ncdf") # Load ncdf library
   trackerfile <- open.ncdf(co2inifile, write=F)
   ctcenterlats <- trackerfile$dim$latitude$vals            # Center
   ctcenterlons <- trackerfile$dim$longitude$vals           # Center
   ctlatres <- unique(diff(ctcenterlats))
   ctlonres <- unique(diff(ctcenterlons))
   ctnumlat <- length(ctcenterlats)
   ctnumlon <- length(ctcenterlons)
   ctlowerleftlats <- as.numeric(ctcenterlats)-0.5*ctlatres
   ctlowerleftlons <- as.numeric(ctcenterlons)-0.5*ctlonres
   hysig.coeffa <- get.var.ncdf(trackerfile, varid="at")    # hybrid sigma coordinate coefficients
   hysig.coeffb <- get.var.ncdf(trackerfile, varid="bt")
   ctnumlevs <- length(hysig.coeffa)-1                      # Coefficients denote breaks
   ctStartDate  <- as.integer(att.get.ncdf(trackerfile,varid=0,attname="StartDate")$value) # expect YYYYMMDD
   ctStartYear  <- ctStartDate %/% 10000
   ctStartMonth <- (ctStartDate %/% 100) %% 100
   ctStartDay   <- ctStartDate %% 100
   ctEndDate    <- as.integer(att.get.ncdf(trackerfile,varid=0,attname="EndDate")$value)
   ctEndYear    <- ctEndDate %/% 10000

   # Get Ending positions
   p4ct <- result[result.sel, "pres"]*100                   # mb to Pa (stilt output in mb; carbontracker uses Pa)
   lat4ct <- result[result.sel, "lat"]
   lon4ct <- result[result.sel, "lon"]


   #-- check reference dates for CT access and reset year to CT range, if necessary

   refdate <- month.day.year(julian(mon, day, yr4) + hr/24 - result[result.sel, "btime"]/24)
   refdate$year <- pmin(pmax(refdate$year,ctStartYear), ctEndYear)
   if (trackerfile$dim$date$units != "days since 2000-01-01")
      stop("getCarbonTracker: wrong date unit in CarbonTracker file. Stop.") # cttimeindices formula wrong
   cttimeindices <- floor(julian(refdate$month, refdate$day, refdate$year)
                          - julian(ctStartMonth,ctStartDay,ctStartYear)) + 1
   ctboundary <- rep(NA, length(cttimeindices))


   # loop over unique end days
   for (jjct in unique(cttimeindices)) {

      ctsel <- which(cttimeindices == jjct)
      sublat4ct <- lat4ct[ctsel]
      sublon4ct <- lon4ct[ctsel]
      subp4ct <- p4ct[ctsel]

      trackersurfp <- get.var.ncdf(trackerfile, varid="psurf", start=c(1,1, jjct),
                                   count=c(ctnumlon, ctnumlat,1))

      # Make Pointer
      ctlatlimlow <- min(ctlowerleftlats)
      ctlatlimhi <- max(ctlowerleftlats)+unique(diff(ctlowerleftlats))
      ctlonlimlow <- min(ctlowerleftlons)
      ctlonlimhi <- max(ctlowerleftlons)+unique(diff(ctlowerleftlons))

      ctlatpointer <- findInterval(sublat4ct, c(ctlowerleftlats, ctlatlimhi), rightmost.closed=TRUE,all.inside=TRUE)
      ctlonpointer <- findInterval(sublon4ct, c(ctlowerleftlons, ctlonlimhi), rightmost.closed=TRUE,all.inside=TRUE)

      # Vertical pressure pointer will vary with surface pressure due to hybrid-sigma coordinates, loop through
      ctprespointer <- rep(0, length(ctlatpointer))
      for (ppct in 1:length(subp4ct)) {
         trackerpresbreaks <- hysig.coeffa+hysig.coeffb*trackersurfp[ctlonpointer[ppct], ctlatpointer[ppct]]
         if (subp4ct[ppct]>max(trackerpresbreaks))
            ctprespointer[ppct] <- 1
         else
            ctprespointer[ppct] <- findInterval(-subp4ct[ppct],-trackerpresbreaks,
                                                rightmost.closed=TRUE,all.inside=TRUE)
      } # for ppct

      ctpointer <- cbind(ctlonpointer, ctlatpointer, ctprespointer)

      # Get CO2
      trackerco2 <- get.var.ncdf(trackerfile, varid="co2mf", start=c(1,1,1, jjct),
                                 count=c(ctnumlon, ctnumlat, ctnumlevs,1))

      # Select CO2 from array
      ctboundary[ctsel] <- trackerco2[ctpointer]
   }

   result[,"co2ini"] <- rep(0,nrow(result))
   result[result.sel,"co2ini"] <- ctboundary

   close.ncdf(trackerfile)

   return(result)

}
